@@ -16,7 +16,7 @@ from AnyQt.QtWidgets import (
     QGraphicsProxyWidget, QGraphicsItemGroup, QGraphicsSimpleTextItem,
     QGraphicsRectItem, QFrame, QSizePolicy
 )
-from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QFont, QFontMetrics
+from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QFontMetrics
 from AnyQt.QtCore import Qt, QEvent, QRectF, QSizeF, QSize, QPointF
 from AnyQt.QtCore import pyqtSignal as Signal
 
@@ -32,7 +32,10 @@ from Orange.widgets.utils.annotated_data import (create_annotated_table,
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.unsupervised.owhierarchicalclustering import \
     WrapperLayoutItem
-from Orange.widgets.widget import Msg
+from Orange.widgets.widget import Msg, Input, Output
+
+
+ROW_NAMES_WIDTH = 200
 
 
 class OWSilhouettePlot(widget.OWWidget):
@@ -43,9 +46,12 @@ class OWSilhouettePlot(widget.OWWidget):
     icon = "icons/SilhouettePlot.svg"
     priority = 300
 
-    inputs = [("Data", Orange.data.Table, "set_data")]
-    outputs = [("Selected Data", Orange.data.Table, widget.Default),
-               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
+    class Inputs:
+        data = Input("Data", Orange.data.Table)
+
+    class Outputs:
+        selected_data = Output("Selected Data", Orange.data.Table, default=True)
+        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)
 
     replaces = [
         "orangecontrib.prototypes.widgets.owsilhouetteplot.OWSilhouettePlot",
@@ -77,6 +83,7 @@ class OWSilhouettePlot(widget.OWWidget):
     class Error(widget.OWWidget.Error):
         need_two_clusters = Msg("Need at least two non-empty clusters")
         singleton_clusters_all = Msg("All clusters are singletons")
+        memory_error = Msg("Not enough memory")
 
     class Warning(widget.OWWidget.Warning):
         missing_cluster_assignment = Msg(
@@ -157,6 +164,7 @@ class OWSilhouettePlot(widget.OWWidget):
         return sh.expandedTo(QSize(600, 720))
 
     @check_sql_input
+    @Inputs.data
     def set_data(self, data):
         """
         Set the input data set.
@@ -246,18 +254,36 @@ class OWSilhouettePlot(widget.OWWidget):
 
     def _update(self):
         # Update/recompute the distances/scores as required
+        self._clear_messages()
+
         if self.data is None or not len(self.data):
-            self._mask = None
-            self._silhouette = None
-            self._labels = None
-            self._matrix = None
-            self._clear_scene()
+            self._reset_all()
             return
 
         if self._matrix is None and self._effective_data is not None:
             _, metric = self.Distances[self.distance_idx]
-            self._matrix = numpy.asarray(metric(self._effective_data))
+            try:
+                self._matrix = numpy.asarray(metric(self._effective_data))
+            except MemoryError:
+                self.Error.memory_error()
+                return
 
+        self._update_labels()
+
+    def _reset_all(self):
+        self._mask = None
+        self._silhouette = None
+        self._labels = None
+        self._matrix = None
+        self._clear_scene()
+
+    def _clear_messages(self):
+        self.Error.memory_error.clear()
+        self.Error.singleton_clusters_all.clear()
+        self.Error.need_two_clusters.clear()
+        self.Warning.missing_cluster_assignment.clear()
+
+    def _update_labels(self):
         labelvar = self.cluster_var_model[self.cluster_var_idx]
         labels, _ = self.data.get_column_view(labelvar)
         labels = numpy.asarray(labels, dtype=float)
@@ -265,11 +291,7 @@ class OWSilhouettePlot(widget.OWWidget):
         labels = labels.astype(int)
         labels = labels[~mask]
 
-        labels_unq, counts = numpy.unique(labels, return_counts=True)
-
-        self.Error.singleton_clusters_all.clear()
-        self.Error.need_two_clusters.clear()
-        self.Warning.missing_cluster_assignment.clear()
+        labels_unq, _ = numpy.unique(labels, return_counts=True)
 
         if len(labels_unq) < 2:
             self.Error.need_two_clusters()
@@ -381,8 +403,7 @@ class OWSilhouettePlot(widget.OWWidget):
                     self.data.domain.attributes,
                     self.data.domain.class_vars,
                     self.data.domain.metas + (silhouette_var, ))
-                data = self.data.from_table(
-                    domain, self.data)
+                data = self.data.transform(domain)
             else:
                 domain = self.data.domain
                 data = self.data
@@ -396,9 +417,8 @@ class OWSilhouettePlot(widget.OWWidget):
                     selected[:, silhouette_var] = numpy.c_[scores[selectedmask]]
                 data[:, silhouette_var] = numpy.c_[scores]
 
-        self.send("Selected Data", selected)
-        self.send(ANNOTATED_DATA_SIGNAL_NAME,
-                  create_annotated_table(data, indices))
+        self.Outputs.selected_data.send(selected)
+        self.Outputs.annotated_data.send(create_annotated_table(data, indices))
 
     def send_report(self):
         if not len(self.cluster_var_model):
@@ -507,7 +527,10 @@ class SilhouettePlot(QGraphicsWidget):
             item = layout.itemAt(i + 1, 3)
 
             if grp.rownames is not None:
-                item.setItems(grp.rownames)
+                metrics = QFontMetrics(self.font())
+                rownames = [metrics.elidedText(rowname, Qt.ElideRight, ROW_NAMES_WIDTH)
+                            for rowname in grp.rownames]
+                item.setItems(rownames)
                 item.setVisible(self.__rowNamesVisible)
             else:
                 item.setItems([])
@@ -521,8 +544,8 @@ class SilhouettePlot(QGraphicsWidget):
             else:
                 tooltips = grp.rownames
 
-            for bar, tooltip in zip(baritems, tooltips):
-                bar.setToolTip(tooltip)
+            for baritem, tooltip in zip(baritems, tooltips):
+                baritem.setToolTip(tooltip)
 
         self.layout().activate()
 
@@ -907,8 +930,7 @@ class BarPlotItem(QGraphicsWidget):
         return super().event(event)
 
     def sizeHint(self, which, constraint=QSizeF()):
-        spacing = max(self.__spacing * (self.count() - 1), 0)
-        return QSizeF(300, self.__barsize * self.count() + spacing)
+        return QSizeF(300, (self.__barsize + self.__spacing) * self.count())
 
     def setPreferredBarSize(self, size):
         if self.__barsize != size:
@@ -918,11 +940,6 @@ class BarPlotItem(QGraphicsWidget):
     def spacing(self):
         return self.__spacing
 
-    def setSpacing(self, spacing):
-        if self.__spacing != spacing:
-            self.__spacing = spacing
-            self.updateGeometry()
-
     def setPen(self, pen):
         pen = QPen(pen)
         if self.__pen != pen:
@@ -967,7 +984,7 @@ class BarPlotItem(QGraphicsWidget):
 
         pen = self.pen()
         brush = self.brush()
-        for i in range(self.count()):
+        for _ in range(self.count()):
             item = QGraphicsRectItem(self)
             item.setPen(pen)
             item.setBrush(brush)
@@ -1015,7 +1032,7 @@ class TextListWidget(QGraphicsWidget):
         self.__spacing = 0
 
         sp = QSizePolicy(QSizePolicy.Preferred,
-                               QSizePolicy.Preferred)
+                         QSizePolicy.Preferred)
         sp.setWidthForHeight(True)
         self.setSizePolicy(sp)
 
@@ -1111,8 +1128,10 @@ class TextListWidget(QGraphicsWidget):
         self.__textitems = []
 
 
-def main(argv=sys.argv):
+def main(argv=None):
     from AnyQt.QtWidgets import QApplication
+    if argv is None:
+        argv = sys.argv
     app = QApplication(list(argv))
     argv = app.arguments()
     if len(argv) > 1:
