@@ -24,12 +24,14 @@ from AnyQt.QtCore import Qt, QPointF, QRectF, QLineF
 from AnyQt.QtCore import pyqtSignal as Signal
 
 import Orange.data
+import Orange.statistics.util as util
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import itemmodels, colorpalette
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_NAME)
 from Orange.widgets.io import FileFormat
 from Orange.widgets.utils.sql import check_sql_input
+from Orange.widgets.widget import Input, Output
 
 
 _InputData = namedtuple("_InputData", ["key", "name", "table"])
@@ -43,9 +45,12 @@ class OWVennDiagram(widget.OWWidget):
     icon = "icons/VennDiagram.svg"
     priority = 280
 
-    inputs = [("Data", Orange.data.Table, "setData", widget.Multiple)]
-    outputs = [("Selected Data", Orange.data.Table, widget.Default),
-               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
+    class Inputs:
+        data = Input("Data", Orange.data.Table, multiple=True)
+
+    class Outputs:
+        selected_data = Output("Selected Data", Orange.data.Table, default=True)
+        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)
 
     # Selected disjoint subset indices
     selection = settings.Setting([])
@@ -139,6 +144,7 @@ class OWVennDiagram(widget.OWWidget):
         self._queue = []
 
     @check_sql_input
+    @Inputs.data
     def setData(self, data, key=None):
         self.error()
         if not self._inputUpdate:
@@ -174,7 +180,7 @@ class OWVennDiagram(widget.OWWidget):
         has_identifiers = all(source_attributes(input.table.domain)
                               for input in self.data.values())
         has_any_identifiers = any(source_attributes(input.table.domain)
-                              for input in self.data.values())
+                                  for input in self.data.values())
         self.useequalityButton.setEnabled(samedomain)
         self.useidentifiersButton.setEnabled(
             has_any_identifiers or len(self.data) == 0)
@@ -494,7 +500,7 @@ class OWVennDiagram(widget.OWWidget):
                 index = i
                 break
 
-        assert (index is not None)
+        assert index is not None
 
         key, _ = inputs[index]
 
@@ -596,7 +602,7 @@ class OWVennDiagram(widget.OWWidget):
             data = table_concat(selected_subsets)
             # Get all variables which are not constant between the same
             # item set
-            varying = varying_between(data, [item_id_var])
+            varying = varying_between(data, item_id_var)
 
             if source_var in varying:
                 varying.remove(source_var)
@@ -613,15 +619,15 @@ class OWVennDiagram(widget.OWWidget):
             annotated_data = table_concat(annotated_data_subsets)
             indices = numpy.hstack(annotated_data_masks)
             annotated_data = create_annotated_table(annotated_data, indices)
-            varying = varying_between(annotated_data, [item_id_var])
+            varying = varying_between(annotated_data, item_id_var)
             if source_var in varying:
                 varying.remove(source_var)
             annotated_data = reshape_wide(annotated_data, varying,
                                           [item_id_var], [source_var])
             annotated_data = drop_columns(annotated_data, [item_id_var])
 
-        self.send("Selected Data", data)
-        self.send(ANNOTATED_DATA_SIGNAL_NAME, annotated_data)
+        self.Outputs.selected_data.send(data)
+        self.Outputs.annotated_data.send(annotated_data)
 
     def getSettings(self, *args, **kwargs):
         self._storeHints()
@@ -713,13 +719,9 @@ def table_concat(tables):
         variables_seen.update(table.domain.metas)
 
     domain = Orange.data.Domain(attributes, class_vars, metas)
-    new_table = Orange.data.Table(domain)
 
-    for table in tables:
-        new_table.extend(Orange.data.Table.from_table(domain, table))
-        new_table.attributes.update(table.attributes)
-
-    return new_table
+    tables = [tab.transform(domain) for tab in tables]
+    return tables[0].concatenate(tables, axis=0)
 
 
 def copy_descriptor(descriptor, newname=None):
@@ -788,10 +790,12 @@ def reshape_wide(table, varlist, idvarlist, groupvarlist):
     # each instance in this list belongs to one group (but not all
     # groups need to be present).
     inst_by_id = defaultdict(list)
+    id_by_inst = defaultdict(list)  # inverse mapping
 
     for i in range(len(table)):
         inst_id = instance_ids[i]
         inst_by_id[inst_id].append(i)
+        id_by_inst[i] = inst_id
 
     newfeatures = []
     newclass_vars = []
@@ -829,21 +833,27 @@ def reshape_wide(table, varlist, idvarlist, groupvarlist):
 
     domain = Orange.data.Domain(newfeatures, newclass_vars, newmetas)
     prototype_indices = [inst_by_id[inst_id][0] for inst_id in ids]
-    newtable = Orange.data.Table.from_table(domain, table)[prototype_indices]
+    newtable = table[prototype_indices].transform(domain)
     in_expanded = set(f for efd in expanded_features.values() for f in efd.values())
 
+    # Fill-in nan values
+    for var in domain.variables + domain.metas:
+        if var in idvarlist or var in in_expanded:
+            continue
+        col, _ = newtable.get_column_view(var)
+        nan_indices = (i for i in col.nonzero()[0]
+                       if isinstance(col[i], str) or numpy.isnan(col[i]))
+        for i in nan_indices:
+            for ind in inst_by_id[ids[i]]:
+                if not numpy.isnan(table[ind, var]):
+                    newtable[i, var] = table[ind, var]
+                    break
+
+    # Fill-in expanded features if any
     for i, inst_id in enumerate(ids):
         indices = inst_by_id[inst_id]
         instance = newtable[i]
 
-        for var in domain.variables + domain.metas:
-            if var in idvarlist or var in in_expanded:
-                continue
-            if numpy.isnan(instance[var]):
-                for ind in indices:
-                    if not numpy.isnan(table[ind, var]):
-                        newtable[i, var] = table[ind, var]
-
         for index in indices:
             source_inst = table[index]
             group = instance_groups[index]
@@ -867,8 +877,6 @@ def unique(seq):
             yield item
             seen.add(item)
 
-from Orange.widgets.data.owmergedata import group_table_indices
-
 
 def unique_non_nan(ar):
     # metas have sometimes object dtype, but values are numpy floats
@@ -877,35 +885,29 @@ def unique_non_nan(ar):
     return uniq[~numpy.isnan(uniq)]
 
 
-def varying_between(table, idvarlist):
+def varying_between(table, idvar):
     """
     Return a list of all variables with non constant values between
-    groups defined by `idvarlist`.
+    groups defined by `idvar`.
 
     """
-    def inst_key(inst, vars):
-        return tuple(str(inst[var]) for var in vars)
-
-    excluded = set(idvarlist)
     all_possible = [var for var in table.domain.variables + table.domain.metas
-                    if var not in excluded]
+                    if var != idvar]
     candidate_set = set(all_possible)
 
-    idmap = group_table_indices(table, idvarlist)
-    values = {}
+    idmap = group_table_indices(table, idvar)
+
     varying = set()
     for indices in idmap.values():
         subset = table[indices]
         for var in list(candidate_set):
-            values = subset[:, var]
-            values, _ = subset.get_column_view(var)
+            column, _ = subset.get_column_view(var)
+            values = util.unique(column)
 
-            if var.is_string:
-                uniq = set(values)
-            else:
-                uniq = unique_non_nan(values)
+            if not var.is_string:
+                values = unique_non_nan(values)
 
-            if len(uniq) > 1:
+            if len(values) > 1:
                 varying.add(var)
                 candidate_set.remove(var)
 
@@ -939,16 +941,14 @@ def string_attributes(domain):
     """
     Return all string attributes from the domain.
     """
-    return [attr for attr in domain.variables + domain.metas
-            if attr.is_string]
+    return [attr for attr in domain.variables + domain.metas if attr.is_string]
 
 
 def discrete_attributes(domain):
     """
     Return all discrete attributes from the domain.
     """
-    return [attr for attr in domain.variables + domain.metas
-                 if attr.is_discrete]
+    return [attr for attr in domain.variables + domain.metas if attr.is_discrete]
 
 
 def source_attributes(domain):
@@ -1439,8 +1439,7 @@ def subset_anchors(shapes):
                 unit_point(270, r=0.35),  # C
                 unit_point(210, r=0.27),  # AC
                 unit_point(330, r=0.27),  # BC
-                unit_point(0, r=0),       # ABC
-                ]
+                unit_point(0, r=0),]      # ABC
     elif n == 4:
         anchors = [
             (0.400, 0.110),    # A
@@ -1623,7 +1622,6 @@ def append_column(data, where, variable, column):
     attr = domain.attributes
     class_vars = domain.class_vars
     metas = domain.metas
-
     if where == "X":
         attr = attr + (variable,)
         X = numpy.hstack((X, column))
@@ -1636,10 +1634,9 @@ def append_column(data, where, variable, column):
     else:
         raise ValueError
     domain = Orange.data.Domain(attr, class_vars, metas)
-    table = Orange.data.Table.from_numpy(domain, X, Y, M, W if W.size else None)
-    table.ids = data.ids
-    table.attributes = data.attributes
-    return table
+    new_data = data.transform(domain)
+    new_data[:, variable] = column
+    return new_data
 
 
 def drop_columns(data, columns):
@@ -1656,19 +1653,31 @@ def drop_columns(data, columns):
     return Orange.data.Table.from_table(domain, data)
 
 
+def group_table_indices(table, key_var):
+    """
+    Group table indices based on values of selected columns (`key_vars`).
+
+    Return a dictionary mapping all unique value combinations (keys)
+    into a list of indices in the table where they are present.
+    """
+    groups = defaultdict(list)
+    for i, inst in enumerate(table):
+        groups[str(inst[key_var])].append(i)
+    return groups
+
+
 def test():
-    import sklearn.cross_validation as skl_cross_validation
+    from Orange.evaluation import ShuffleSplit
+
     app = QApplication([])
     w = OWVennDiagram()
     data = Orange.data.Table("brown-selected")
     data = append_column(data, "M", Orange.data.StringVariable("Test"),
                          numpy.arange(len(data)).reshape(-1, 1) % 30)
 
-    indices = skl_cross_validation.ShuffleSplit(
-        len(data), n_iter=5, test_size=0.7
-    )
-
-    indices = iter(indices)
+    res = ShuffleSplit(data, [None], n_resamples=5,
+                       test_size=0.7, stratified=False)
+    indices = iter(res.indices)
 
     def select(data):
         sample, _ = next(indices)
